package main;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

/** <h1>Application</h1>
 * This class contains the main method of this project
 * It reads a the file "expressions.txt" line-by-line and saves the value into a String
 * The syntax and semantic of the mathematical expression will be tested in multiple steps
 * First a Lexer will separate the String into multiple Tokens, then a Parser checks if the
 * Tokens can be merged into an AST(Arithmetical Syntax Tree). If not, a ParserException will
 * be thrown. If the syntax and semantic of the expression was correct, the expression will
 * be calculated by an Evaluator and finally be shown in a FunctionPlotter
 *
 * @version 1.0
 * @since 09.01.2023
 * @see Lexer
 * @see Parser
 * @see ASTPrinter
 * @see Evaluator
 * @see SwingFunctionPlotter
 */
public class Application {
    public static void main(String[] args) {
        //A list that will contain a String for each line of the "expressions.txt" file
        List<String> expressions = new LinkedList<>();
        //Reads the file "expressions.txt" and adds its values to the list expressions
        BufferedReader reader;
        String datapath = "expressions.txt";
        try {
            System.out.println("Reading the file...\n");
            reader = new BufferedReader(new FileReader(datapath));
            String currentLine = reader.readLine();
            while(currentLine !=null) {
                expressions.add(currentLine);
                currentLine = reader.readLine();
            }
            reader.close();
        }
        catch(IOException e) {
            e.printStackTrace();
        }

        LinkedList<String> stringsForPlotter = new LinkedList<>();
        //Iterate through every String
        for (String expressionString: expressions) {
            System.out.println("Expression: " + expressionString);
            //Usage of the lexer, parser & evaluator
            try {
                //Create a list of tokens with the lexer
                List<Lexer.Token> tokens = Lexer.lex(expressionString);
                System.out.println("Lexer created tokens:");
                //prints out each token to the console
                int index = 0;
                for(Lexer.Token token: tokens) {
                    System.out.println("[" + index + "]" + token.toString());
                    index++;
                }
                //Parse all tokens with the parser and saves the result into the variable 'ast'
                Parser parser = new Parser();
                Parser.Expression ast = parser.parse(tokens);

                //Prints the ast out with an ASTPrinter
                //the result can later be used to show the ast in a function plotter
                ASTPrinter printer = new ASTPrinter();
                System.out.println("Printing: '" + printer.visit(ast) + "'");
                stringsForPlotter.add(printer.visit(ast));

                //Prints the Value of the Expression
                System.out.println("Evaluating:");
                Evaluator eval = new Evaluator();
                //Calculates the expression with values from -20 to 20
                for(index=-20;index<=20; index++) {
                    System.out.print("[" + index + "]" + eval.visit(ast,index) + " ");
                    if(index%5==0 && index!=-20) {
                        System.out.println();
                    }
                }
                System.out.println("\n");
            }
            catch (Exception e) {
                e.printStackTrace();
            }
        }
        SwingFunctionPlotter.plotFunction(stringsForPlotter);
    }
}
