package main;

import com.mindfusion.charting.FunctionSeries;
import com.mindfusion.charting.swing.LineChart;

import javax.swing.*;
import java.awt.*;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.LinkedList;

/**
 * <h1>SwingFunctionPlotter</h1>
 * This class creates a FunctionPlotter Panel using JavaSwing.
 * It can display a list of different functions and allows the user to navigate through them
 *
 * @see #plotFunction(LinkedList) 
 */
public class SwingFunctionPlotter extends JFrame {
    private static final long serialVersionUID = 1L;

    /**
     * This function is used to display the functions.
     *  * The LeftArrow Button can be used to navigate to the function shown before, every other
     *  * Button input will navigate to the next function
     * @param function A LinkedList of Strings that contains the function expressions
     */
    public static void plotFunction(LinkedList<String> function) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                try {
                    new SwingFunctionPlotter(function).setVisible(true);
                }
                catch(Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }

    /**
     * This is the Constructor of the FunctionPlotter
     * It will create the FunctionPlotter with all its functions
     * @param function A LinkedList of Strings that contains the function expressions
     */
    private SwingFunctionPlotter(LinkedList<String> function) {
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setSize(650, 400);
        setTitle("Java Swing Window: Informatik III - Exercise 4");

        //Keeps track of the current shown function
        final int[] index = {0};
        getContentPane().add(initializeChart(function.get(0),0), BorderLayout.CENTER);

        //KeyListener that listens to every KeyInput of the user
        addKeyListener(new KeyAdapter() {
            @Override
            public void keyPressed(KeyEvent e) {
                //If the user pressed the left arrow this following block will be entered
                //The function shown before will be shown again
                if(e.getKeyCode() == KeyEvent.VK_LEFT) {
                    if(index[0]>0) {
                        index[0]--;
                        getContentPane().removeAll();
                        getContentPane().add(initializeChart(function.get(index[0]),index[0]), BorderLayout.CENTER);
                        validate();
                        setVisible(true);
                    }
                }
                //If any other key got pressed this following block will be entered
                //The next function will be shown
                else {
                    index[0]++;
                    if(index[0]<function.size()) {
                        getContentPane().removeAll();
                        getContentPane().add(initializeChart(function.get(index[0]),index[0]), BorderLayout.CENTER);
                        validate();
                        setVisible(true);
                    }
                    else {
                        //If the last function was already be shown the FunctionPlotter will close
                        //and 'destroy' itself
                        getContentPane().removeAll();
                        setVisible(false);
                        dispose();
                    }
                }
            }
        });
    }

    /**
     * This function will initialize a FunctionSeries out of the current function that should be shown
     */
    private LineChart initializeChart(String function, int index) {
        LineChart lineChart = new LineChart();
        FunctionSeries series1;
        try {
            series1 = new FunctionSeries(function,1000, -20, 20);
            series1.setTitle("Function " + (index+1) + ": " + function);
            lineChart.getSeries().add(series1);
        }
        catch(Exception e) {
            e.printStackTrace();
        }
        return lineChart;
    }
}