package test;

import main.Lexer;
import main.Parser;
import org.junit.Test;

import java.util.LinkedList;
import java.util.List;

public class ParserTest {

    @Test
    public void testParseVariable() {
        List<Lexer.Token> tokens = new LinkedList<>();
        tokens.add(new Lexer.Token(Lexer.TokenType.VARIABLE,"x"));
        var parser = new Parser();
        try {
            var ast = parser.parse(tokens);
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void testWrongVariableInput() {
        List<Lexer.Token> tokens = new LinkedList<>();
        tokens.add(new Lexer.Token(Lexer.TokenType.VARIABLE,"xy"));
        var parser = new Parser();
        try {
            var ast = parser.parse(tokens);
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void testParseNumber() {
        List<Lexer.Token> tokens = new LinkedList<>();
        tokens.add(new Lexer.Token(Lexer.TokenType.NUMBER,"23"));
        var parser = new Parser();
        try {
            var ast = parser.parse(tokens);
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void testWrongNumberInput() {
        List<Lexer.Token> tokens = new LinkedList<>();
        tokens.add(new Lexer.Token(Lexer.TokenType.NUMBER,"02.4"));
        var parser = new Parser();
        try {
            var ast = parser.parse(tokens);
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void testParseBinaryOperation() {
        List<Lexer.Token> tokens = new LinkedList<>();
        tokens.add(new Lexer.Token(Lexer.TokenType.SPECIAL,"("));
        tokens.add(new Lexer.Token(Lexer.TokenType.NUMBER,"5"));
        tokens.add(new Lexer.Token(Lexer.TokenType.SPECIAL,"*"));
        tokens.add(new Lexer.Token(Lexer.TokenType.NUMBER,"2"));
        tokens.add(new Lexer.Token(Lexer.TokenType.SPECIAL,")"));
        tokens.add(new Lexer.Token(Lexer.TokenType.NUMBER,"*"));
        tokens.add(new Lexer.Token(Lexer.TokenType.NUMBER,"3"));
        var parser = new Parser();
        try {
            var ast = parser.parse(tokens);
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void testParseWrongBinaryOperation() {
        List<Lexer.Token> tokens = new LinkedList<>();
        tokens.add(new Lexer.Token(Lexer.TokenType.SPECIAL,"("));
        tokens.add(new Lexer.Token(Lexer.TokenType.NUMBER,"5"));
        tokens.add(new Lexer.Token(Lexer.TokenType.SPECIAL,"*"));
        tokens.add(new Lexer.Token(Lexer.TokenType.NUMBER,"2"));
        tokens.add(new Lexer.Token(Lexer.TokenType.SPECIAL,")"));
        tokens.add(new Lexer.Token(Lexer.TokenType.NUMBER,"3"));
        var parser = new Parser();
        try {
            var ast = parser.parse(tokens);
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }
}
